"""
RuntimeConfig
=============
"""
from ansys.dpf.core.data_tree import DataTree
from ansys.dpf.core.common import types
from ansys.dpf.core import misc


class _RuntimeConfig:
    """Parent class for configuration options."""

    def __init__(self, data_tree, server=None):
        if isinstance(data_tree, DataTree):
            self._data_tree = data_tree
        else:
            self._data_tree = DataTree(data_tree=data_tree, server=server)


class RuntimeClientConfig(_RuntimeConfig):
    """Enables to access and set runtime configuration
    options to gRPC client. Mostly used to configurate gRPC streaming and calls options.

    Parameters
    ----------
    data_tree: ctypes.cvoid_p
        DataTree pointer describing the existing parameters configuration
        of DataprocessingCore.
    server : BaseServer, optional
        Server with channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Notes
    -----
    Available from 4.0 server version.

    Examples
    --------
    Get runtime configuration for Client.

    >>> from ansys.dpf import core as dpf
    >>> server = dpf.start_local_server(config=dpf.server_factory.AvailableServerConfigs.GrpcServer
    ...    , as_global=False)
    >>> client_config = dpf.settings.get_runtime_client_config(server=server)
    >>> cache_enabled = client_config.cache_enabled
    >>> stream_floats_instead_of_doubles = client_config.stream_floats_instead_of_doubles
    >>> streaming_buffer_size = client_config.streaming_buffer_size
    >>> return_arrays = client_config.return_arrays
    >>> client_config.return_arrays = return_arrays # or True/False
    >>> client_config.cache_enabled = cache_enabled # or True/False
    >>> client_config.stream_floats_instead_of_doubles = stream_floats_instead_of_doubles
    ... # or True/False
    >>> client_config.streaming_buffer_size = streaming_buffer_size # or 10000, 2000 ...
    """

    def __init__(self, data_tree, server=None):
        super().__init__(data_tree=data_tree, server=server)
        if not self._data_tree.has("return_arrays"):
            self._data_tree.add(return_arrays=int(misc.RETURN_ARRAYS))

    @property
    def cache_enabled(self):
        """Whether gRPC requests and responses are intercepted
        to cache them and retrieve them when appropriate.

        Returns
        -------
        bool

        Notes
        -----
        Can only be used for a gRPC communication.

        """
        return bool(self._data_tree.get_as("use_cache", types.int))

    @cache_enabled.setter
    def cache_enabled(self, value):
        self._data_tree.add(use_cache=int(value))

    @property
    def streaming_buffer_size(self):
        """Sets the chunk size (in bytes) used in gRPC streaming calls.

        Returns
        -------
        int

        Notes
        -----
        Can only be used for a gRPC communication.

        """
        return self._data_tree.get_as("streaming_buffer_size", types.int)

    @streaming_buffer_size.setter
    def streaming_buffer_size(self, value):
        self._data_tree.add(streaming_buffer_size=int(value))

    @property
    def stream_floats_instead_of_doubles(self):
        """Sets whether double values (8 bytes) should be converted and streamed as float values
         (4 bytes) in gRPC streaming calls.

        Returns
        -------
        bool

        Notes
        -----
        Can only be used for a gRPC communication.

        """
        return bool(self._data_tree.get_as("stream_floats", types.int))

    @stream_floats_instead_of_doubles.setter
    def stream_floats_instead_of_doubles(self, value):
        self._data_tree.add(stream_floats=int(value))

    @property
    def return_arrays(self):
        """All methods will return :class:`ansys.dpf.core.DPFArray` (instead of lists)
        when possible. Default is ``True``.
        See for example, :func:`ansys.dpf.core.Scoping.ids`.

        Returns
        -------
        bool
        """
        return bool(self._data_tree.get_as("return_arrays", types.int))

    @return_arrays.setter
    def return_arrays(self, value):
        self._data_tree.add(return_arrays=int(value))


class RuntimeCoreConfig(_RuntimeConfig):
    """Enables to access and set runtime configuration
    options to DataProcessingCore.

    Parameters
    ----------
    data_tree: ctypes.cvoid_p
        DataTree pointer describing the existing parameters configuration
        of DataprocessingCore.
    server : BaseServer, optional
        Server with channel connected to the remote or local instance.
        The default is ``None``, in which case an attempt is made to use the
        global server.

    Notes
    -----
    Available from 4.0 server version.

    Examples
    --------
    Get runtime configuration for DataProcessingCore.

    >>> from ansys.dpf import core as dpf
    >>> server = dpf.start_local_server(config=dpf.server_factory.AvailableServerConfigs.GrpcServer
    ...    , as_global=False) # doctest: +SKIP
    >>> core_config = dpf.settings.get_runtime_core_config(server=server) # doctest: +SKIP
    >>> num_threads = core_config.num_threads # doctest: +SKIP
    >>> core_config.num_threads = num_threads # or 3, 6, ... # doctest: +SKIP
    """

    def __init__(self, data_tree, server=None):
        super().__init__(data_tree=data_tree, server=server)

    @property
    def num_threads(self):
        """Sets the default number of threads to use for all operators,
        default is omp_get_num_threads.

        Returns
        -------
        int
        """
        return self._data_tree.get_as("num_threads", types.int)

    @num_threads.setter
    def num_threads(self, value):
        self._data_tree.add(num_threads=int(value))

    @property
    def license_timeout_in_seconds(self):
        """Sets the default number of threads to use for all operators,
        default is omp_get_num_threads.

        Returns
        -------
        float
        """
        return self._data_tree.get_as("license_timeout_in_seconds", types.double)

    @license_timeout_in_seconds.setter
    def license_timeout_in_seconds(self, value):
        self._data_tree.add(license_timeout_in_seconds=float(value))



"""
for_each
========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class for_each(Operator):
    """Allows to write a loop over a chunk of operators.

    Parameters
    ----------
    iterable :
        Either the result of the make_iterable_info
        operator, or the operator that must
        be incremented.
    iterable_values : optional
    pin_index : int, optional
    forward1 :
    forward2 :


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.for_each()

    >>> # Make input connections
    >>> my_iterable = dpf.()
    >>> op.inputs.iterable.connect(my_iterable)
    >>> my_iterable_values = dpf.()
    >>> op.inputs.iterable_values.connect(my_iterable_values)
    >>> my_pin_index = int()
    >>> op.inputs.pin_index.connect(my_pin_index)
    >>> my_forward1 = dpf.()
    >>> op.inputs.forward1.connect(my_forward1)
    >>> my_forward2 = dpf.()
    >>> op.inputs.forward2.connect(my_forward2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.for_each(
    ...     iterable=my_iterable,
    ...     iterable_values=my_iterable_values,
    ...     pin_index=my_pin_index,
    ...     forward1=my_forward1,
    ...     forward2=my_forward2,
    ... )

    >>> # Get output data
    >>> result_empty = op.outputs.empty()
    >>> result_output1 = op.outputs.output1()
    >>> result_output2 = op.outputs.output2()
    """

    def __init__(
        self,
        iterable=None,
        iterable_values=None,
        pin_index=None,
        forward1=None,
        forward2=None,
        config=None,
        server=None,
    ):
        super().__init__(name="for_each", config=config, server=server)
        self._inputs = InputsForEach(self)
        self._outputs = OutputsForEach(self)
        if iterable is not None:
            self.inputs.iterable.connect(iterable)
        if iterable_values is not None:
            self.inputs.iterable_values.connect(iterable_values)
        if pin_index is not None:
            self.inputs.pin_index.connect(pin_index)
        if forward1 is not None:
            self.inputs.forward1.connect(forward1)
        if forward2 is not None:
            self.inputs.forward2.connect(forward2)

    @staticmethod
    def _spec():
        description = """Allows to write a loop over a chunk of operators."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="iterable",
                    type_names=["any"],
                    optional=False,
                    document="""Either the result of the make_iterable_info
        operator, or the operator that must
        be incremented.""",
                ),
                1: PinSpecification(
                    name="iterable_values",
                    type_names=["any"],
                    optional=True,
                    document="""""",
                ),
                2: PinSpecification(
                    name="pin_index",
                    type_names=["int32"],
                    optional=True,
                    document="""""",
                ),
                3: PinSpecification(
                    name="forward",
                    type_names=["any"],
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="forward",
                    type_names=["any"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="empty",
                    optional=False,
                    document="""""",
                ),
                3: PinSpecification(
                    name="output1",
                    optional=False,
                    document="""""",
                ),
                4: PinSpecification(
                    name="output2",
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="for_each", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsForEach
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsForEach
        """
        return super().outputs


class InputsForEach(_Inputs):
    """Intermediate class used to connect user inputs to
    for_each operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.for_each()
    >>> my_iterable = dpf.()
    >>> op.inputs.iterable.connect(my_iterable)
    >>> my_iterable_values = dpf.()
    >>> op.inputs.iterable_values.connect(my_iterable_values)
    >>> my_pin_index = int()
    >>> op.inputs.pin_index.connect(my_pin_index)
    >>> my_forward1 = dpf.()
    >>> op.inputs.forward1.connect(my_forward1)
    >>> my_forward2 = dpf.()
    >>> op.inputs.forward2.connect(my_forward2)
    """

    def __init__(self, op: Operator):
        super().__init__(for_each._spec().inputs, op)
        self._iterable = Input(for_each._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._iterable)
        self._iterable_values = Input(for_each._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._iterable_values)
        self._pin_index = Input(for_each._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._pin_index)
        self._forward1 = Input(for_each._spec().input_pin(3), 3, op, 0)
        self._inputs.append(self._forward1)
        self._forward2 = Input(for_each._spec().input_pin(4), 4, op, 1)
        self._inputs.append(self._forward2)

    @property
    def iterable(self):
        """Allows to connect iterable input to the operator.

        Either the result of the make_iterable_info
        operator, or the operator that must
        be incremented.

        Parameters
        ----------
        my_iterable :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> op.inputs.iterable.connect(my_iterable)
        >>> # or
        >>> op.inputs.iterable(my_iterable)
        """
        return self._iterable

    @property
    def iterable_values(self):
        """Allows to connect iterable_values input to the operator.

        Parameters
        ----------
        my_iterable_values :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> op.inputs.iterable_values.connect(my_iterable_values)
        >>> # or
        >>> op.inputs.iterable_values(my_iterable_values)
        """
        return self._iterable_values

    @property
    def pin_index(self):
        """Allows to connect pin_index input to the operator.

        Parameters
        ----------
        my_pin_index : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> op.inputs.pin_index.connect(my_pin_index)
        >>> # or
        >>> op.inputs.pin_index(my_pin_index)
        """
        return self._pin_index

    @property
    def forward1(self):
        """Allows to connect forward1 input to the operator.

        Parameters
        ----------
        my_forward1 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> op.inputs.forward1.connect(my_forward1)
        >>> # or
        >>> op.inputs.forward1(my_forward1)
        """
        return self._forward1

    @property
    def forward2(self):
        """Allows to connect forward2 input to the operator.

        Parameters
        ----------
        my_forward2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> op.inputs.forward2.connect(my_forward2)
        >>> # or
        >>> op.inputs.forward2(my_forward2)
        """
        return self._forward2


class OutputsForEach(_Outputs):
    """Intermediate class used to get outputs from
    for_each operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.for_each()
    >>> # Connect inputs : op.inputs. ...
    >>> result_empty = op.outputs.empty()
    >>> result_output1 = op.outputs.output1()
    >>> result_output2 = op.outputs.output2()
    """

    def __init__(self, op: Operator):
        super().__init__(for_each._spec().outputs, op)
        self._empty = Output(for_each._spec().output_pin(0), 0, op)
        self._outputs.append(self._empty)
        self._output1 = Output(for_each._spec().output_pin(3), 3, op)
        self._outputs.append(self._output1)
        self._output2 = Output(for_each._spec().output_pin(4), 4, op)
        self._outputs.append(self._output2)

    @property
    def empty(self):
        """Allows to get empty output of the operator

        Returns
        ----------
        my_empty :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> # Connect inputs : op.inputs. ...
        >>> result_empty = op.outputs.empty()
        """  # noqa: E501
        return self._empty

    @property
    def output1(self):
        """Allows to get output1 output of the operator

        Returns
        ----------
        my_output1 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output1 = op.outputs.output1()
        """  # noqa: E501
        return self._output1

    @property
    def output2(self):
        """Allows to get output2 output of the operator

        Returns
        ----------
        my_output2 :

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.for_each()
        >>> # Connect inputs : op.inputs. ...
        >>> result_output2 = op.outputs.output2()
        """  # noqa: E501
        return self._output2



"""
mesh_clip
=========
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mesh_clip(Operator):
    """Clips a volume mesh along an iso value x, and construct the volume
    mesh defined by v < x.

    Parameters
    ----------
    field : Field
    iso_value : float
        Iso value
    closed_surface : int
        1: closed surface, 0: iso surface.
    mesh : MeshedRegion, optional
    slice_surfaces : bool
        True: slicing will also take into account
        shell and skin elements. false:
        slicing will ignore shell and skin
        elements. the default is true.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.mesh_clip()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_iso_value = float()
    >>> op.inputs.iso_value.connect(my_iso_value)
    >>> my_closed_surface = int()
    >>> op.inputs.closed_surface.connect(my_closed_surface)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_slice_surfaces = bool()
    >>> op.inputs.slice_surfaces.connect(my_slice_surfaces)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.mesh_clip(
    ...     field=my_field,
    ...     iso_value=my_iso_value,
    ...     closed_surface=my_closed_surface,
    ...     mesh=my_mesh,
    ...     slice_surfaces=my_slice_surfaces,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    >>> result_mesh = op.outputs.mesh()
    """

    def __init__(
        self,
        field=None,
        iso_value=None,
        closed_surface=None,
        mesh=None,
        slice_surfaces=None,
        config=None,
        server=None,
    ):
        super().__init__(name="mesh_clip", config=config, server=server)
        self._inputs = InputsMeshClip(self)
        self._outputs = OutputsMeshClip(self)
        if field is not None:
            self.inputs.field.connect(field)
        if iso_value is not None:
            self.inputs.iso_value.connect(iso_value)
        if closed_surface is not None:
            self.inputs.closed_surface.connect(closed_surface)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if slice_surfaces is not None:
            self.inputs.slice_surfaces.connect(slice_surfaces)

    @staticmethod
    def _spec():
        description = """Clips a volume mesh along an iso value x, and construct the volume
            mesh defined by v &lt; x."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="iso_value",
                    type_names=["double"],
                    optional=False,
                    document="""Iso value""",
                ),
                2: PinSpecification(
                    name="closed_surface",
                    type_names=["int32"],
                    optional=False,
                    document="""1: closed surface, 0: iso surface.""",
                ),
                3: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""""",
                ),
                4: PinSpecification(
                    name="slice_surfaces",
                    type_names=["bool"],
                    optional=False,
                    document="""True: slicing will also take into account
        shell and skin elements. false:
        slicing will ignore shell and skin
        elements. the default is true.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="mesh",
                    type_names=["meshed_region"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mesh_clip", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshClip
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshClip
        """
        return super().outputs


class InputsMeshClip(_Inputs):
    """Intermediate class used to connect user inputs to
    mesh_clip operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_clip()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_iso_value = float()
    >>> op.inputs.iso_value.connect(my_iso_value)
    >>> my_closed_surface = int()
    >>> op.inputs.closed_surface.connect(my_closed_surface)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_slice_surfaces = bool()
    >>> op.inputs.slice_surfaces.connect(my_slice_surfaces)
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_clip._spec().inputs, op)
        self._field = Input(mesh_clip._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._iso_value = Input(mesh_clip._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._iso_value)
        self._closed_surface = Input(mesh_clip._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._closed_surface)
        self._mesh = Input(mesh_clip._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._mesh)
        self._slice_surfaces = Input(mesh_clip._spec().input_pin(4), 4, op, -1)
        self._inputs.append(self._slice_surfaces)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Parameters
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def iso_value(self):
        """Allows to connect iso_value input to the operator.

        Iso value

        Parameters
        ----------
        my_iso_value : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> op.inputs.iso_value.connect(my_iso_value)
        >>> # or
        >>> op.inputs.iso_value(my_iso_value)
        """
        return self._iso_value

    @property
    def closed_surface(self):
        """Allows to connect closed_surface input to the operator.

        1: closed surface, 0: iso surface.

        Parameters
        ----------
        my_closed_surface : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> op.inputs.closed_surface.connect(my_closed_surface)
        >>> # or
        >>> op.inputs.closed_surface(my_closed_surface)
        """
        return self._closed_surface

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def slice_surfaces(self):
        """Allows to connect slice_surfaces input to the operator.

        True: slicing will also take into account
        shell and skin elements. false:
        slicing will ignore shell and skin
        elements. the default is true.

        Parameters
        ----------
        my_slice_surfaces : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> op.inputs.slice_surfaces.connect(my_slice_surfaces)
        >>> # or
        >>> op.inputs.slice_surfaces(my_slice_surfaces)
        """
        return self._slice_surfaces


class OutputsMeshClip(_Outputs):
    """Intermediate class used to get outputs from
    mesh_clip operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_clip()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    >>> result_mesh = op.outputs.mesh()
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_clip._spec().outputs, op)
        self._field = Output(mesh_clip._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)
        self._mesh = Output(mesh_clip._spec().output_pin(2), 2, op)
        self._outputs.append(self._mesh)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field

    @property
    def mesh(self):
        """Allows to get mesh output of the operator

        Returns
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_clip()
        >>> # Connect inputs : op.inputs. ...
        >>> result_mesh = op.outputs.mesh()
        """  # noqa: E501
        return self._mesh



"""
scoping_get_attribute
=====================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.outputs import _modify_output_spec_with_one_type
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class scoping_get_attribute(Operator):
    """Uses the Scoping APIs to return a given attribute of the scoping in
    input.

    Parameters
    ----------
    scoping : Scoping
    property_name : str
        Supported property names are: "ids",
        "location".


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.scoping.scoping_get_attribute()

    >>> # Make input connections
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.scoping.scoping_get_attribute(
    ...     scoping=my_scoping,
    ...     property_name=my_property_name,
    ... )

    >>> # Get output data
    >>> result_property = op.outputs.property()
    """

    def __init__(self, scoping=None, property_name=None, config=None, server=None):
        super().__init__(name="scoping::get_attribute", config=config, server=server)
        self._inputs = InputsScopingGetAttribute(self)
        self._outputs = OutputsScopingGetAttribute(self)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if property_name is not None:
            self.inputs.property_name.connect(property_name)

    @staticmethod
    def _spec():
        description = """Uses the Scoping APIs to return a given attribute of the scoping in
            input."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="property_name",
                    type_names=["string"],
                    optional=False,
                    document="""Supported property names are: "ids",
        "location".""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="property",
                    type_names=["vector<int32>", "string"],
                    optional=False,
                    document="""Returns a vector of int for property: "ids"
        and a string for property:
        "location".""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="scoping::get_attribute", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsScopingGetAttribute
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsScopingGetAttribute
        """
        return super().outputs


class InputsScopingGetAttribute(_Inputs):
    """Intermediate class used to connect user inputs to
    scoping_get_attribute operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.scoping_get_attribute()
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_property_name = str()
    >>> op.inputs.property_name.connect(my_property_name)
    """

    def __init__(self, op: Operator):
        super().__init__(scoping_get_attribute._spec().inputs, op)
        self._scoping = Input(scoping_get_attribute._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._scoping)
        self._property_name = Input(
            scoping_get_attribute._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._property_name)

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator.

        Parameters
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.scoping_get_attribute()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def property_name(self):
        """Allows to connect property_name input to the operator.

        Supported property names are: "ids",
        "location".

        Parameters
        ----------
        my_property_name : str

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.scoping.scoping_get_attribute()
        >>> op.inputs.property_name.connect(my_property_name)
        >>> # or
        >>> op.inputs.property_name(my_property_name)
        """
        return self._property_name


class OutputsScopingGetAttribute(_Outputs):
    """Intermediate class used to get outputs from
    scoping_get_attribute operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.scoping.scoping_get_attribute()
    >>> # Connect inputs : op.inputs. ...
    >>> result_property = op.outputs.property()
    """

    def __init__(self, op: Operator):
        super().__init__(scoping_get_attribute._spec().outputs, op)
        self.property_as_vector_int32_ = Output(
            _modify_output_spec_with_one_type(
                scoping_get_attribute._spec().output_pin(0), "vector_int32_"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_vector_int32_)
        self.property_as_string = Output(
            _modify_output_spec_with_one_type(
                scoping_get_attribute._spec().output_pin(0), "string"
            ),
            0,
            op,
        )
        self._outputs.append(self.property_as_string)



import ansys.dpf.core as dpf
import numpy as np


def animate_mode(
    fields_container,
    mode_number=1,
    type_mode=0,
    frame_number=None,
    save_as="",
    deform_scale_factor=1.0,
    **kwargs,
):
    # other option: instead of `type` use `min_factor` and `max_factor`.

    """Creates a modal animation based on Fields contained in the FieldsContainer.

    This method creates a movie or a gif based on the time ids of a ``FieldsContainer``.
    For kwargs see pyvista.Plotter.open_movie/add_text/show.

    Parameters
    ----------
    field_container :
        Field container containing the modal results.
    mode_number : int, optional
        Mode number of the results to animation. The default is ``1``.
    type_mode : int, optional
        Whether it is 0 or 1. Default to 0.
        If 0, the norm of the displacements will be scaled from 1 to -1 to 1.
        If 1, the norm of the displacements will be scaled between -1 and 1.
    save_as : Path of file to save the animation to. Defaults to None. Can be of any format
        supported by pyvista.Plotter.write_frame (.gif, .mp4, ...).
    deform_scale_factor : float, optional
        Scale factor to apply when warping the mesh. Defaults to 1.0.

    Examples
    --------
    Import a modal result from a model.

    >>> import ansys.dpf.core as dpf
    >>> from ansys.dpf.core import examples
    >>> model = dpf.Model(examples.download_modal_frame())
    >>> disp = model.results.displacement.on_all_time_freqs.eval()

    Creates an animation from a modal result.

    >>> from ansys.dpf.core import animation
    >>> animation.animate_mode(disp, mode_number=1, save_as="tmp.gif")


    """
    from ansys.dpf.core.animator import Animator

    # Animation type

    if type_mode == 1:
        if frame_number is None:
            frame_number = 21
        scale_factor_per_frame = list(abs(np.linspace(-1, 1, frame_number, dtype=np.double)))
    elif type_mode == 0:
        if frame_number is None:
            frame_number = 41
        elif frame_number % 2 == 0:
            frame_number -= 1
        half_scale = np.linspace(-1, 1, int((frame_number + 1) / 2), dtype=np.double)
        scale_factor_per_frame = np.concatenate([np.flip(half_scale), half_scale[1:]])
    else:
        raise ValueError(
            f"The type_mode {type_mode} is not accepted. "
            + "Please select one in 'positive_disp' and 'full_disp'."
        )

    # Get fields
    fields_mode = fields_container.get_fields({"time": mode_number})

    # Merge fields if needed
    if len(fields_mode) > 1:
        merge_op = dpf.operators.utility.merge_fields()
        for i, field in enumerate(fields_mode):
            merge_op.connect(i, field)
        field_mode = merge_op.eval()
    else:
        field_mode = fields_mode[0]

    max_data = float(np.max(field_mode.data))
    loop_over = dpf.fields_factory.field_from_array(scale_factor_per_frame)

    # Create workflow
    wf = dpf.Workflow()
    wf.progress_bar = False

    # Add scaling operator
    scaling_op = dpf.operators.math.scale()
    scaling_op.inputs.field.connect(field_mode)
    wf.add_operators([scaling_op])

    wf.set_input_name("ponderation", scaling_op.inputs.ponderation)
    wf.set_output_name("field", scaling_op.outputs.field)

    anim = Animator(workflow=wf, **kwargs)

    return anim.animate(
        loop_over=loop_over,
        input_name="ponderation",
        output_name="field",
        save_as=save_as,
        mode_number=mode_number,
        clim=[0, max_data],
        **kwargs,
    )



"""
members_in_bending_not_certified
================================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class members_in_bending_not_certified(Operator):
    """This operator is a non-certified example of buckling resistance
    verification for the bending members. It is only provided as an
    example if you want to develop your own compute norm operator. The
    results computed by this beta operator have not been certified by
    ANSYS. ANSYS declines all responsibility for the use of this
    operator. HATS Beam and irregular beams (unequal I-Beam, not-
    square Channel-Beam, not-square Angle L-beam, unequal hollow
    rectangular beam) not supported.

    Parameters
    ----------
    time_scoping : Scoping or int, optional
    field_yield_strength : Field
        This pin contains field of beam's yield
        strength defined by the user.
    class_cross_section : bool
        Selection for a cross-section. true: class 1
        or 2 cross-sections. false: class 3
        cross section. if the user defines
        the cross section as class 1 or 2,
        the section modulus would be plastic
        section modulus. if it's class 3-
        cross section,the section modulus
        would be elastic section modulus
    streams : StreamsContainer, optional
        result file container allowed to be kept open
        to cache data.
    data_sources : DataSources, optional
        Result file path container, used if no
        streams are set.
    partial_factor : float
        Partial safety factor for resistance of
        members to instability assessed by
        member checks. default value: 1.
    mesh : MeshedRegion
        mesh containing beam's properties defined by
        user
    bending_moment_y : FieldsContainer
        Fields container of bending moment on axis y
        defined by user
    bending_moment_z : FieldsContainer
        Fields container of bending moment on axis z
        defined by user


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.result.members_in_bending_not_certified()

    >>> # Make input connections
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_field_yield_strength = dpf.Field()
    >>> op.inputs.field_yield_strength.connect(my_field_yield_strength)
    >>> my_class_cross_section = bool()
    >>> op.inputs.class_cross_section.connect(my_class_cross_section)
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_partial_factor = float()
    >>> op.inputs.partial_factor.connect(my_partial_factor)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_bending_moment_y = dpf.FieldsContainer()
    >>> op.inputs.bending_moment_y.connect(my_bending_moment_y)
    >>> my_bending_moment_z = dpf.FieldsContainer()
    >>> op.inputs.bending_moment_z.connect(my_bending_moment_z)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.result.members_in_bending_not_certified(
    ...     time_scoping=my_time_scoping,
    ...     field_yield_strength=my_field_yield_strength,
    ...     class_cross_section=my_class_cross_section,
    ...     streams=my_streams,
    ...     data_sources=my_data_sources,
    ...     partial_factor=my_partial_factor,
    ...     mesh=my_mesh,
    ...     bending_moment_y=my_bending_moment_y,
    ...     bending_moment_z=my_bending_moment_z,
    ... )

    >>> # Get output data
    >>> result_buckling_resistance_bending_yy = op.outputs.buckling_resistance_bending_yy()
    >>> result_buckling_resistance_bending_zz = op.outputs.buckling_resistance_bending_zz()
    """

    def __init__(
        self,
        time_scoping=None,
        field_yield_strength=None,
        class_cross_section=None,
        streams=None,
        data_sources=None,
        partial_factor=None,
        mesh=None,
        bending_moment_y=None,
        bending_moment_z=None,
        config=None,
        server=None,
    ):
        super().__init__(
            name="members_in_bending_not_certified", config=config, server=server
        )
        self._inputs = InputsMembersInBendingNotCertified(self)
        self._outputs = OutputsMembersInBendingNotCertified(self)
        if time_scoping is not None:
            self.inputs.time_scoping.connect(time_scoping)
        if field_yield_strength is not None:
            self.inputs.field_yield_strength.connect(field_yield_strength)
        if class_cross_section is not None:
            self.inputs.class_cross_section.connect(class_cross_section)
        if streams is not None:
            self.inputs.streams.connect(streams)
        if data_sources is not None:
            self.inputs.data_sources.connect(data_sources)
        if partial_factor is not None:
            self.inputs.partial_factor.connect(partial_factor)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if bending_moment_y is not None:
            self.inputs.bending_moment_y.connect(bending_moment_y)
        if bending_moment_z is not None:
            self.inputs.bending_moment_z.connect(bending_moment_z)

    @staticmethod
    def _spec():
        description = """This operator is a non-certified example of buckling resistance
            verification for the bending members. It is only provided
            as an example if you want to develop your own compute norm
            operator. The results computed by this beta operator have
            not been certified by ANSYS. ANSYS declines all
            responsibility for the use of this operator. HATS Beam and
            irregular beams (unequal I-Beam, not-square Channel-Beam,
            not-square Angle L-beam, unequal hollow rectangular beam)
            not supported."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_scoping",
                    type_names=["scoping", "vector<int32>", "int32"],
                    optional=True,
                    document="""""",
                ),
                1: PinSpecification(
                    name="field_yield_strength",
                    type_names=["field"],
                    optional=False,
                    document="""This pin contains field of beam's yield
        strength defined by the user.""",
                ),
                2: PinSpecification(
                    name="class_cross_section",
                    type_names=["bool"],
                    optional=False,
                    document="""Selection for a cross-section. true: class 1
        or 2 cross-sections. false: class 3
        cross section. if the user defines
        the cross section as class 1 or 2,
        the section modulus would be plastic
        section modulus. if it's class 3-
        cross section,the section modulus
        would be elastic section modulus""",
                ),
                3: PinSpecification(
                    name="streams",
                    type_names=["streams_container"],
                    optional=True,
                    document="""result file container allowed to be kept open
        to cache data.""",
                ),
                4: PinSpecification(
                    name="data_sources",
                    type_names=["data_sources"],
                    optional=True,
                    document="""Result file path container, used if no
        streams are set.""",
                ),
                6: PinSpecification(
                    name="partial_factor",
                    type_names=["double"],
                    optional=False,
                    document="""Partial safety factor for resistance of
        members to instability assessed by
        member checks. default value: 1.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""mesh containing beam's properties defined by
        user""",
                ),
                8: PinSpecification(
                    name="bending_moment_y",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fields container of bending moment on axis y
        defined by user""",
                ),
                9: PinSpecification(
                    name="bending_moment_z",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fields container of bending moment on axis z
        defined by user""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="buckling_resistance_bending_yy",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fields container of buckling resistance
        factor on axis y-y in case of
        bending(m). these factors should be
        less than 1 and positive.""",
                ),
                1: PinSpecification(
                    name="buckling_resistance_bending_zz",
                    type_names=["fields_container"],
                    optional=False,
                    document="""Fields container of buckling resistance
        factor on axis z-z in case of
        bending(m). these factors should be
        less than 1 and positive.""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(
            name="members_in_bending_not_certified", server=server
        )

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMembersInBendingNotCertified
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMembersInBendingNotCertified
        """
        return super().outputs


class InputsMembersInBendingNotCertified(_Inputs):
    """Intermediate class used to connect user inputs to
    members_in_bending_not_certified operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.members_in_bending_not_certified()
    >>> my_time_scoping = dpf.Scoping()
    >>> op.inputs.time_scoping.connect(my_time_scoping)
    >>> my_field_yield_strength = dpf.Field()
    >>> op.inputs.field_yield_strength.connect(my_field_yield_strength)
    >>> my_class_cross_section = bool()
    >>> op.inputs.class_cross_section.connect(my_class_cross_section)
    >>> my_streams = dpf.StreamsContainer()
    >>> op.inputs.streams.connect(my_streams)
    >>> my_data_sources = dpf.DataSources()
    >>> op.inputs.data_sources.connect(my_data_sources)
    >>> my_partial_factor = float()
    >>> op.inputs.partial_factor.connect(my_partial_factor)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_bending_moment_y = dpf.FieldsContainer()
    >>> op.inputs.bending_moment_y.connect(my_bending_moment_y)
    >>> my_bending_moment_z = dpf.FieldsContainer()
    >>> op.inputs.bending_moment_z.connect(my_bending_moment_z)
    """

    def __init__(self, op: Operator):
        super().__init__(members_in_bending_not_certified._spec().inputs, op)
        self._time_scoping = Input(
            members_in_bending_not_certified._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._time_scoping)
        self._field_yield_strength = Input(
            members_in_bending_not_certified._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._field_yield_strength)
        self._class_cross_section = Input(
            members_in_bending_not_certified._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._class_cross_section)
        self._streams = Input(
            members_in_bending_not_certified._spec().input_pin(3), 3, op, -1
        )
        self._inputs.append(self._streams)
        self._data_sources = Input(
            members_in_bending_not_certified._spec().input_pin(4), 4, op, -1
        )
        self._inputs.append(self._data_sources)
        self._partial_factor = Input(
            members_in_bending_not_certified._spec().input_pin(6), 6, op, -1
        )
        self._inputs.append(self._partial_factor)
        self._mesh = Input(
            members_in_bending_not_certified._spec().input_pin(7), 7, op, -1
        )
        self._inputs.append(self._mesh)
        self._bending_moment_y = Input(
            members_in_bending_not_certified._spec().input_pin(8), 8, op, -1
        )
        self._inputs.append(self._bending_moment_y)
        self._bending_moment_z = Input(
            members_in_bending_not_certified._spec().input_pin(9), 9, op, -1
        )
        self._inputs.append(self._bending_moment_z)

    @property
    def time_scoping(self):
        """Allows to connect time_scoping input to the operator.

        Parameters
        ----------
        my_time_scoping : Scoping or int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.time_scoping.connect(my_time_scoping)
        >>> # or
        >>> op.inputs.time_scoping(my_time_scoping)
        """
        return self._time_scoping

    @property
    def field_yield_strength(self):
        """Allows to connect field_yield_strength input to the operator.

        This pin contains field of beam's yield
        strength defined by the user.

        Parameters
        ----------
        my_field_yield_strength : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.field_yield_strength.connect(my_field_yield_strength)
        >>> # or
        >>> op.inputs.field_yield_strength(my_field_yield_strength)
        """
        return self._field_yield_strength

    @property
    def class_cross_section(self):
        """Allows to connect class_cross_section input to the operator.

        Selection for a cross-section. true: class 1
        or 2 cross-sections. false: class 3
        cross section. if the user defines
        the cross section as class 1 or 2,
        the section modulus would be plastic
        section modulus. if it's class 3-
        cross section,the section modulus
        would be elastic section modulus

        Parameters
        ----------
        my_class_cross_section : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.class_cross_section.connect(my_class_cross_section)
        >>> # or
        >>> op.inputs.class_cross_section(my_class_cross_section)
        """
        return self._class_cross_section

    @property
    def streams(self):
        """Allows to connect streams input to the operator.

        result file container allowed to be kept open
        to cache data.

        Parameters
        ----------
        my_streams : StreamsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.streams.connect(my_streams)
        >>> # or
        >>> op.inputs.streams(my_streams)
        """
        return self._streams

    @property
    def data_sources(self):
        """Allows to connect data_sources input to the operator.

        Result file path container, used if no
        streams are set.

        Parameters
        ----------
        my_data_sources : DataSources

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.data_sources.connect(my_data_sources)
        >>> # or
        >>> op.inputs.data_sources(my_data_sources)
        """
        return self._data_sources

    @property
    def partial_factor(self):
        """Allows to connect partial_factor input to the operator.

        Partial safety factor for resistance of
        members to instability assessed by
        member checks. default value: 1.

        Parameters
        ----------
        my_partial_factor : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.partial_factor.connect(my_partial_factor)
        >>> # or
        >>> op.inputs.partial_factor(my_partial_factor)
        """
        return self._partial_factor

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        mesh containing beam's properties defined by
        user

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def bending_moment_y(self):
        """Allows to connect bending_moment_y input to the operator.

        Fields container of bending moment on axis y
        defined by user

        Parameters
        ----------
        my_bending_moment_y : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.bending_moment_y.connect(my_bending_moment_y)
        >>> # or
        >>> op.inputs.bending_moment_y(my_bending_moment_y)
        """
        return self._bending_moment_y

    @property
    def bending_moment_z(self):
        """Allows to connect bending_moment_z input to the operator.

        Fields container of bending moment on axis z
        defined by user

        Parameters
        ----------
        my_bending_moment_z : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> op.inputs.bending_moment_z.connect(my_bending_moment_z)
        >>> # or
        >>> op.inputs.bending_moment_z(my_bending_moment_z)
        """
        return self._bending_moment_z


class OutputsMembersInBendingNotCertified(_Outputs):
    """Intermediate class used to get outputs from
    members_in_bending_not_certified operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.result.members_in_bending_not_certified()
    >>> # Connect inputs : op.inputs. ...
    >>> result_buckling_resistance_bending_yy = op.outputs.buckling_resistance_bending_yy()
    >>> result_buckling_resistance_bending_zz = op.outputs.buckling_resistance_bending_zz()
    """

    def __init__(self, op: Operator):
        super().__init__(members_in_bending_not_certified._spec().outputs, op)
        self._buckling_resistance_bending_yy = Output(
            members_in_bending_not_certified._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._buckling_resistance_bending_yy)
        self._buckling_resistance_bending_zz = Output(
            members_in_bending_not_certified._spec().output_pin(1), 1, op
        )
        self._outputs.append(self._buckling_resistance_bending_zz)

    @property
    def buckling_resistance_bending_yy(self):
        """Allows to get buckling_resistance_bending_yy output of the operator

        Returns
        ----------
        my_buckling_resistance_bending_yy : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> # Connect inputs : op.inputs. ...
        >>> result_buckling_resistance_bending_yy = op.outputs.buckling_resistance_bending_yy()
        """  # noqa: E501
        return self._buckling_resistance_bending_yy

    @property
    def buckling_resistance_bending_zz(self):
        """Allows to get buckling_resistance_bending_zz output of the operator

        Returns
        ----------
        my_buckling_resistance_bending_zz : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.result.members_in_bending_not_certified()
        >>> # Connect inputs : op.inputs. ...
        >>> result_buckling_resistance_bending_zz = op.outputs.buckling_resistance_bending_zz()
        """  # noqa: E501
        return self._buckling_resistance_bending_zz



"""
elements_volumes_over_time
==========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class elements_volumes_over_time(Operator):
    """Calculates for a mesh, the volume of each element over time for each
    specified time step.

    Parameters
    ----------
    scoping : Scoping, optional
    displacement : FieldsContainer, optional
        Displacement field's container. must contain
        the mesh if mesh not specified in
        input.
    mesh : MeshedRegion, optional
        Mesh must be defined if the displacement
        field's container does not contain
        it, or if there is no displacement.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.geo.elements_volumes_over_time()

    >>> # Make input connections
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_displacement = dpf.FieldsContainer()
    >>> op.inputs.displacement.connect(my_displacement)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.geo.elements_volumes_over_time(
    ...     scoping=my_scoping,
    ...     displacement=my_displacement,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self, scoping=None, displacement=None, mesh=None, config=None, server=None
    ):
        super().__init__(name="volumes_provider", config=config, server=server)
        self._inputs = InputsElementsVolumesOverTime(self)
        self._outputs = OutputsElementsVolumesOverTime(self)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if displacement is not None:
            self.inputs.displacement.connect(displacement)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Calculates for a mesh, the volume of each element over time for each
            specified time step."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""""",
                ),
                2: PinSpecification(
                    name="displacement",
                    type_names=["fields_container"],
                    optional=True,
                    document="""Displacement field's container. must contain
        the mesh if mesh not specified in
        input.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""Mesh must be defined if the displacement
        field's container does not contain
        it, or if there is no displacement.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="volumes_provider", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsElementsVolumesOverTime
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsElementsVolumesOverTime
        """
        return super().outputs


class InputsElementsVolumesOverTime(_Inputs):
    """Intermediate class used to connect user inputs to
    elements_volumes_over_time operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.elements_volumes_over_time()
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_displacement = dpf.FieldsContainer()
    >>> op.inputs.displacement.connect(my_displacement)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(elements_volumes_over_time._spec().inputs, op)
        self._scoping = Input(
            elements_volumes_over_time._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._scoping)
        self._displacement = Input(
            elements_volumes_over_time._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._displacement)
        self._mesh = Input(elements_volumes_over_time._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator.

        Parameters
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_volumes_over_time()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def displacement(self):
        """Allows to connect displacement input to the operator.

        Displacement field's container. must contain
        the mesh if mesh not specified in
        input.

        Parameters
        ----------
        my_displacement : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_volumes_over_time()
        >>> op.inputs.displacement.connect(my_displacement)
        >>> # or
        >>> op.inputs.displacement(my_displacement)
        """
        return self._displacement

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh must be defined if the displacement
        field's container does not contain
        it, or if there is no displacement.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_volumes_over_time()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsElementsVolumesOverTime(_Outputs):
    """Intermediate class used to get outputs from
    elements_volumes_over_time operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.elements_volumes_over_time()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(elements_volumes_over_time._spec().outputs, op)
        self._fields_container = Output(
            elements_volumes_over_time._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.elements_volumes_over_time()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
component_transformer_fc
========================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class component_transformer_fc(Operator):
    """Takes the input field and creates a field with overriden value on
    given components.

    Parameters
    ----------
    fields_container : FieldsContainer
    component_number : int
        One or several component index that will be
        modified from the initial field.
    default_value : float, optional
        Set a default value for components selected.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.logic.component_transformer_fc()

    >>> # Make input connections
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_component_number = int()
    >>> op.inputs.component_number.connect(my_component_number)
    >>> my_default_value = float()
    >>> op.inputs.default_value.connect(my_default_value)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.logic.component_transformer_fc(
    ...     fields_container=my_fields_container,
    ...     component_number=my_component_number,
    ...     default_value=my_default_value,
    ... )

    >>> # Get output data
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(
        self,
        fields_container=None,
        component_number=None,
        default_value=None,
        config=None,
        server=None,
    ):
        super().__init__(name="component_transformer_fc", config=config, server=server)
        self._inputs = InputsComponentTransformerFc(self)
        self._outputs = OutputsComponentTransformerFc(self)
        if fields_container is not None:
            self.inputs.fields_container.connect(fields_container)
        if component_number is not None:
            self.inputs.component_number.connect(component_number)
        if default_value is not None:
            self.inputs.default_value.connect(default_value)

    @staticmethod
    def _spec():
        description = """Takes the input field and creates a field with overriden value on
            given components."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="component_number",
                    type_names=["int32", "vector<int32>"],
                    optional=False,
                    document="""One or several component index that will be
        modified from the initial field.""",
                ),
                2: PinSpecification(
                    name="default_value",
                    type_names=["double"],
                    optional=True,
                    document="""Set a default value for components selected.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="fields_container",
                    type_names=["fields_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="component_transformer_fc", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsComponentTransformerFc
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsComponentTransformerFc
        """
        return super().outputs


class InputsComponentTransformerFc(_Inputs):
    """Intermediate class used to connect user inputs to
    component_transformer_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.component_transformer_fc()
    >>> my_fields_container = dpf.FieldsContainer()
    >>> op.inputs.fields_container.connect(my_fields_container)
    >>> my_component_number = int()
    >>> op.inputs.component_number.connect(my_component_number)
    >>> my_default_value = float()
    >>> op.inputs.default_value.connect(my_default_value)
    """

    def __init__(self, op: Operator):
        super().__init__(component_transformer_fc._spec().inputs, op)
        self._fields_container = Input(
            component_transformer_fc._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._fields_container)
        self._component_number = Input(
            component_transformer_fc._spec().input_pin(1), 1, op, -1
        )
        self._inputs.append(self._component_number)
        self._default_value = Input(
            component_transformer_fc._spec().input_pin(2), 2, op, -1
        )
        self._inputs.append(self._default_value)

    @property
    def fields_container(self):
        """Allows to connect fields_container input to the operator.

        Parameters
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.component_transformer_fc()
        >>> op.inputs.fields_container.connect(my_fields_container)
        >>> # or
        >>> op.inputs.fields_container(my_fields_container)
        """
        return self._fields_container

    @property
    def component_number(self):
        """Allows to connect component_number input to the operator.

        One or several component index that will be
        modified from the initial field.

        Parameters
        ----------
        my_component_number : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.component_transformer_fc()
        >>> op.inputs.component_number.connect(my_component_number)
        >>> # or
        >>> op.inputs.component_number(my_component_number)
        """
        return self._component_number

    @property
    def default_value(self):
        """Allows to connect default_value input to the operator.

        Set a default value for components selected.

        Parameters
        ----------
        my_default_value : float

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.component_transformer_fc()
        >>> op.inputs.default_value.connect(my_default_value)
        >>> # or
        >>> op.inputs.default_value(my_default_value)
        """
        return self._default_value


class OutputsComponentTransformerFc(_Outputs):
    """Intermediate class used to get outputs from
    component_transformer_fc operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.logic.component_transformer_fc()
    >>> # Connect inputs : op.inputs. ...
    >>> result_fields_container = op.outputs.fields_container()
    """

    def __init__(self, op: Operator):
        super().__init__(component_transformer_fc._spec().outputs, op)
        self._fields_container = Output(
            component_transformer_fc._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._fields_container)

    @property
    def fields_container(self):
        """Allows to get fields_container output of the operator

        Returns
        ----------
        my_fields_container : FieldsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.logic.component_transformer_fc()
        >>> # Connect inputs : op.inputs. ...
        >>> result_fields_container = op.outputs.fields_container()
        """  # noqa: E501
        return self._fields_container



"""
gauss_to_node
=============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class gauss_to_node(Operator):
    """Extrapolating results available at Gauss or quadrature points to nodal
    points for one field. The available elements are: Linear
    quadrangle, parabolic quadrangle, linear hexagonal, quadratic
    hexagonal, linear tetrahedral, and quadratic tetrahedral

    Parameters
    ----------
    field : Field
    scoping : Scoping, optional
        Scoping to integrate on, if not provided, the
        one from input field is provided.
    mesh : MeshedRegion, optional
        Mesh to integrate on.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.geo.gauss_to_node()

    >>> # Make input connections
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.geo.gauss_to_node(
    ...     field=my_field,
    ...     scoping=my_scoping,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_field = op.outputs.field()
    """

    def __init__(self, field=None, scoping=None, mesh=None, config=None, server=None):
        super().__init__(name="gauss_to_node", config=config, server=server)
        self._inputs = InputsGaussToNode(self)
        self._outputs = OutputsGaussToNode(self)
        if field is not None:
            self.inputs.field.connect(field)
        if scoping is not None:
            self.inputs.scoping.connect(scoping)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Extrapolating results available at Gauss or quadrature points to nodal
            points for one field. The available elements are: Linear
            quadrangle, parabolic quadrangle, linear hexagonal,
            quadratic hexagonal, linear tetrahedral, and quadratic
            tetrahedral"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=True,
                    document="""Scoping to integrate on, if not provided, the
        one from input field is provided.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=True,
                    document="""Mesh to integrate on.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="gauss_to_node", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsGaussToNode
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsGaussToNode
        """
        return super().outputs


class InputsGaussToNode(_Inputs):
    """Intermediate class used to connect user inputs to
    gauss_to_node operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.gauss_to_node()
    >>> my_field = dpf.Field()
    >>> op.inputs.field.connect(my_field)
    >>> my_scoping = dpf.Scoping()
    >>> op.inputs.scoping.connect(my_scoping)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(gauss_to_node._spec().inputs, op)
        self._field = Input(gauss_to_node._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._field)
        self._scoping = Input(gauss_to_node._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scoping)
        self._mesh = Input(gauss_to_node._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def field(self):
        """Allows to connect field input to the operator.

        Parameters
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.gauss_to_node()
        >>> op.inputs.field.connect(my_field)
        >>> # or
        >>> op.inputs.field(my_field)
        """
        return self._field

    @property
    def scoping(self):
        """Allows to connect scoping input to the operator.

        Scoping to integrate on, if not provided, the
        one from input field is provided.

        Parameters
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.gauss_to_node()
        >>> op.inputs.scoping.connect(my_scoping)
        >>> # or
        >>> op.inputs.scoping(my_scoping)
        """
        return self._scoping

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Mesh to integrate on.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.gauss_to_node()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsGaussToNode(_Outputs):
    """Intermediate class used to get outputs from
    gauss_to_node operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.geo.gauss_to_node()
    >>> # Connect inputs : op.inputs. ...
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(gauss_to_node._spec().outputs, op)
        self._field = Output(gauss_to_node._spec().output_pin(0), 0, op)
        self._outputs.append(self._field)

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.geo.gauss_to_node()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field



import numpy as np
import pyvista as pv
import ansys.dpf.core as dpf

from ansys.dpf.core import errors
from vtk import (
    VTK_HEXAHEDRON,
    VTK_LINE,
    VTK_POLYGON,
    VTK_POLYHEDRON,
    VTK_PYRAMID,
    VTK_QUAD,
    VTK_QUADRATIC_EDGE,
    VTK_QUADRATIC_HEXAHEDRON,
    VTK_QUADRATIC_POLYGON,
    VTK_QUADRATIC_PYRAMID,
    VTK_QUADRATIC_QUAD,
    VTK_QUADRATIC_TETRA,
    VTK_QUADRATIC_TRIANGLE,
    VTK_TETRA,
    VTK_TRIANGLE,
    VTK_VERTEX,
    VTK_WEDGE,
    vtkVersion,
)
from ansys.dpf.core.elements import element_types

VTK9 = vtkVersion().GetVTKMajorVersion() >= 9


# DPF --> VTK mapping
# any cells not mapped will be empty cells
VTK_MAPPING = np.array(
    [
        VTK_QUADRATIC_TETRA,  # kAnsTet10 = 0,
        VTK_QUADRATIC_HEXAHEDRON,  # kAnsHex20 = 1,
        # VTK_QUADRATIC_WEDGE,  # kAnsWedge15 = 2,
        VTK_WEDGE,  # kAnsWedge15 = 2,
        VTK_QUADRATIC_PYRAMID,  # kAnsPyramid13 = 3,
        VTK_QUADRATIC_TRIANGLE,  # kAnsTri6 = 4,
        VTK_QUADRATIC_TRIANGLE,  # kAnsTriShell6 = 5,
        VTK_QUADRATIC_QUAD,  # kAnsQuad8 = 6,
        VTK_QUADRATIC_QUAD,  # kAnsQuadShell8 = 7,
        VTK_QUADRATIC_EDGE,  # kAnsLine3 = 8,
        VTK_VERTEX,  # kAnsPoint1 = 9,
        VTK_TETRA,  # kAnsTet4 = 10,
        VTK_HEXAHEDRON,  # kAnsHex8 = 11,
        VTK_WEDGE,  # kAnsWedge6 = 12,
        VTK_PYRAMID,  # kAnsPyramid5 = 13,
        VTK_TRIANGLE,  # kAnsTri3 = 14,
        VTK_TRIANGLE,  # kAnsTriShell3 = 15,
        VTK_QUAD,  # kAnsQuad4 = 16,
        VTK_QUAD,  # kAnsQuadShell4 = 17,
        VTK_LINE,  # kAnsLine2 = 18,
        0,  # kAnsNumElementTypes = 19,
        0,  # kAnsUnknown = 20,
        0,  # kAnsEMagLine = 21,
        0,  # kAnsEMagArc = 22,
        0,  # kAnsEMagCircle = 23,
        VTK_TRIANGLE,  # kAnsSurface3 = 24,
        VTK_QUAD,  # kAnsSurface4 = 25,
        VTK_QUADRATIC_TRIANGLE,  # kAnsSurface6 = 26,
        VTK_QUADRATIC_QUAD,  # kAnsSurface8 = 27,
        0,  # kAnsEdge2 = 28,
        0,  # kAnsEdge3 = 29,
        0,  # kAnsBeam3 = 30,
        0,  # kAnsBeam4 = 31,
        0,  # kAnsGeneralPlaceholder = 32,
        VTK_QUADRATIC_POLYGON,  # kAnsPolygon = 33,
        VTK_POLYHEDRON,  # kAnsPolyhedron = 34,
    ]
)


# map all cells to linear
VTK_LINEAR_MAPPING = np.array(
    [
        VTK_TETRA,  # kAnsTet10 = 0,
        VTK_HEXAHEDRON,  # kAnsHex20 = 1,
        VTK_WEDGE,  # kAnsWedge15 = 2,
        VTK_PYRAMID,  # kAnsPyramid13 = 3,
        VTK_TRIANGLE,  # kAnsTri6 = 4,
        VTK_TRIANGLE,  # kAnsTriShell6 = 5,
        VTK_QUAD,  # kAnsQuad8 = 6,
        VTK_QUAD,  # kAnsQuadShell8 = 7,
        VTK_LINE,  # kAnsLine3 = 8,
        VTK_VERTEX,  # kAnsPoint1 = 9,
        VTK_TETRA,  # kAnsTet4 = 10,
        VTK_HEXAHEDRON,  # kAnsHex8 = 11,
        VTK_WEDGE,  # kAnsWedge6 = 12,
        VTK_PYRAMID,  # kAnsPyramid5 = 13,
        VTK_TRIANGLE,  # kAnsTri3 = 14,
        VTK_TRIANGLE,  # kAnsTriShell3 = 15,
        VTK_QUAD,  # kAnsQuad4 = 16,
        VTK_QUAD,  # kAnsQuadShell4 = 17,
        VTK_LINE,  # kAnsLine2 = 18,
        0,  # kAnsNumElementTypes = 19,
        0,  # kAnsUnknown = 20,
        0,  # kAnsEMagLine = 21,
        0,  # kAnsEMagArc = 22,
        0,  # kAnsEMagCircle = 23,
        VTK_TRIANGLE,  # kAnsSurface3 = 24,
        VTK_QUAD,  # kAnsSurface4 = 25,
        VTK_QUADRATIC_TRIANGLE,  # kAnsSurface6 = 26,
        VTK_QUADRATIC_QUAD,  # kAnsSurface8 = 27,
        0,  # kAnsEdge2 = 28,
        0,  # kAnsEdge3 = 29,
        0,  # kAnsBeam3 = 30,
        0,  # kAnsBeam4 = 31,
        0,  # kAnsGeneralPlaceholder = 32,
        VTK_POLYGON,  # kAnsPolygon = 33,
        VTK_POLYHEDRON,  # kAnsPolyhedron = 34,
    ]
)


class PyVistaImportError(ModuleNotFoundError):
    """Error raised when PyVista could not be imported during plotting."""

    def __init__(
        self,
        msg="To use plotting capabilities, please install pyvista "
        "with :\n pip install pyvista>=0.32.0",
    ):
        ModuleNotFoundError.__init__(self, msg)


def dpf_mesh_to_vtk_op(mesh, nodes, as_linear):
    """Return a pyvista unstructured grid given DPF node and element
    definitions from operators (server > 6.2)

    Parameters
    ----------
    mesh : dpf.MeshedRegion
        Meshed Region to export to pyVista format

    nodes : dpf.Field
        Field containing the nodes of the mesh.

    as_linear : bool
        Export quadratic surface elements as linear.

    Returns
    -------
    grid : pyvista.UnstructuredGrid
        Unstructured grid of the DPF mesh.
    """
    mesh_to_pyvista = dpf.operators.mesh.mesh_to_pyvista(server=mesh._server)
    mesh_to_pyvista.inputs.mesh.connect(mesh)
    mesh_to_pyvista.inputs.as_linear.connect(as_linear)
    mesh_to_pyvista.inputs.vtk_updated.connect(VTK9)
    if nodes is not None:
        mesh_to_pyvista.inputs.coordinates.connect(nodes)

    nodes_pv = mesh_to_pyvista.outputs.nodes().data
    cells_pv = mesh_to_pyvista.outputs.cells()
    celltypes_pv = mesh_to_pyvista.outputs.cell_types()
    if VTK9:
        grid = pv.UnstructuredGrid(cells_pv, celltypes_pv, nodes_pv)
        setattr(grid, "_dpf_cache_op", [cells_pv, celltypes_pv, nodes_pv])
        return grid
    else:
        offsets_pv = mesh_to_pyvista.outputs.offsets()
        grid = pv.UnstructuredGrid(offsets_pv, cells_pv, celltypes_pv, nodes_pv)
        setattr(grid, "_dpf_cache_op", [cells_pv, celltypes_pv, nodes_pv, offsets_pv])
        return grid


def dpf_mesh_to_vtk_py(mesh, nodes, as_linear):
    """Return a pyvista unstructured grid given DPF node and element
    definitions in pure Python (server <= 6.2)

    Parameters
    ----------
    mesh : dpf.MeshedRegion
        Meshed Region to export to pyVista format

    nodes : dpf.Field
        Field containing the node coordinates of the mesh.

    as_linear : bool
        Export quadratic surface elements as linear.

    Returns
    -------
    grid : pyvista.UnstructuredGrid
        Unstructured grid of the DPF mesh.
    """
    etypes = mesh.elements.element_types_field.data
    connectivity = mesh.elements.connectivities_field
    if nodes is None:
        coordinates_field = mesh.nodes.coordinates_field
        node_coordinates = coordinates_field.data
    else:
        coordinates_field = nodes
        node_coordinates = nodes.data

    elem_size = np.ediff1d(np.append(connectivity._data_pointer, connectivity.shape))

    faces_nodes_connectivity = mesh.property_field("faces_nodes_connectivity")
    faces_nodes_connectivity_dp = np.append(
        faces_nodes_connectivity._data_pointer, len(faces_nodes_connectivity)
    )
    elements_faces_connectivity = mesh.property_field("elements_faces_connectivity")
    elements_faces_connectivity_dp = np.append(
        elements_faces_connectivity._data_pointer, len(elements_faces_connectivity)
    )

    insert_ind = np.cumsum(elem_size)
    insert_ind = np.hstack(([0], insert_ind))[:-1]

    # if not as_linear:
    #     # Pre-handle semi-parabolic elements
    #     semi_mask = connectivity.data == -1
    #     if semi_mask.any():
    #         # Modify -1 connectivity values
    #         repeated_data_pointers = connectivity._data_pointer.repeat(repeats=elem_size)
    #         connectivity.data[semi_mask] = connectivity.data[repeated_data_pointers[semi_mask]]

    # partition cells in vtk format
    cells = np.insert(connectivity.data, insert_ind, elem_size)

    # Check if polyhedrons are present
    if element_types.Polyhedron.value in etypes:
        cells = np.array(cells)
        nodes = np.array(node_coordinates)
        insert_ind = insert_ind + np.asarray(list(range(len(insert_ind))))
        # Replace in cells values for polyhedron format
        # [NValuesToFollow,
        # NFaces, Face1NPoints, Face1Point1, Face1Point2..., Face1PointN, FaceNNPoints,...]]
        for i, ind in reversed(list(enumerate(insert_ind))):
            # Check if this is a polyhedron
            if etypes[i] == element_types.Polyhedron.value:
                # Construct the connectivity for the poly element
                poly_connectivity = []
                faces = elements_faces_connectivity.data[
                    elements_faces_connectivity_dp[i] : elements_faces_connectivity_dp[i + 1]
                ]
                for face in faces:
                    face_connectivity = faces_nodes_connectivity.data[
                        faces_nodes_connectivity_dp[face] : faces_nodes_connectivity_dp[face + 1]
                    ]
                    face_fmt = [len(face_connectivity)] + list(face_connectivity)
                    poly_connectivity += face_fmt
                polyhedron = [len(faces)] + poly_connectivity
                polyhedron = [len(polyhedron)] + polyhedron
                # Replace the whole sequence between this index and the next
                r = list(range(ind, ind + elem_size[i] + 1))
                cells = np.delete(cells, r)
                cells = np.insert(cells, ind, polyhedron)

    def compute_offset():
        """Return the starting point of a cell in the cells array"""
        return insert_ind + np.arange(insert_ind.size)

    cells_insert_ind = compute_offset()
    # convert kAns to VTK cell type
    offset = None
    if as_linear:
        # Map the vtk_cell_type to linear versions of the initial elements types
        vtk_cell_type = VTK_LINEAR_MAPPING[etypes]

        # Create a global mask of connectivity values to take
        mask = np.full(cells.shape, True)

        # Get a mask of quad8 elements in etypes
        quad8_mask = etypes == 6
        # If any quad8
        if np.any(quad8_mask):  # kAnsQuad8
            # Get the starting indices of quad8 elements in cells
            insert_ind_quad8 = cells_insert_ind[quad8_mask]
            # insert_ind_quad8 += np.arange(insert_ind_quad8.size)
            mask[insert_ind_quad8 + 5] = False
            mask[insert_ind_quad8 + 6] = False
            mask[insert_ind_quad8 + 7] = False
            mask[insert_ind_quad8 + 8] = False
            cells[insert_ind_quad8] //= 2

        tri6_mask = etypes == 4  # kAnsTri6 = 4
        if np.any(tri6_mask):
            insert_ind_tri6 = cells_insert_ind[tri6_mask]
            # insert_ind_tri6 += np.arange(insert_ind_tri6.size)
            mask[insert_ind_tri6 + 4] = False
            mask[insert_ind_tri6 + 5] = False
            mask[insert_ind_tri6 + 6] = False
            cells[insert_ind_tri6] //= 2
        cells = cells[mask]

    else:
        vtk_cell_type = VTK_MAPPING[etypes]

        # Handle semi-parabolic elements
        semi_mask = cells == -1
        if semi_mask.any():
            cells_insert_ind = compute_offset()
            # Create a global mask of connectivity values to take
            mask = np.full(cells.shape, True)
            # Build a map of size cells with repeated element beginning index
            repeated_insert_ind = cells_insert_ind.repeat(repeats=elem_size + 1)
            # Apply the semi-mask to get a unique set of indices of semi-parabolic elements in cells
            semi_indices_in_cells = np.array(list(set(repeated_insert_ind[semi_mask])))
            semi_sizes = cells[semi_indices_in_cells]
            semi_quad8 = semi_sizes == 8
            if semi_quad8.any():
                mask[semi_indices_in_cells[semi_quad8] + 5] = False
                mask[semi_indices_in_cells[semi_quad8] + 6] = False
                mask[semi_indices_in_cells[semi_quad8] + 7] = False
                mask[semi_indices_in_cells[semi_quad8] + 8] = False
                cells[semi_indices_in_cells[semi_quad8]] //= 2

                quad8_mask = etypes == 6
                semi_quad8_mask = (cells[cells_insert_ind] == 4) & quad8_mask
                vtk_cell_type[semi_quad8_mask] = VTK_LINEAR_MAPPING[6]
            semi_tri6 = semi_sizes == 6
            if semi_tri6.any():
                mask[semi_indices_in_cells[semi_tri6] + 4] = False
                mask[semi_indices_in_cells[semi_tri6] + 5] = False
                mask[semi_indices_in_cells[semi_tri6] + 6] = False
                cells[semi_indices_in_cells[semi_tri6]] //= 2

                tri6_mask = etypes == 4
                semi_tri6_mask = (cells[cells_insert_ind] == 3) & tri6_mask
                vtk_cell_type[semi_tri6_mask] = VTK_LINEAR_MAPPING[4]
            # Update cells with the mask
            cells = cells[mask]

    # different treatment depending on the version of vtk
    if VTK9:
        # compute offset array when < VTK v9
        grid = pv.UnstructuredGrid(cells, vtk_cell_type, node_coordinates)

        # Quick fix required to hold onto the data as PyVista does not make a copy.
        # All of those now return DPFArrays
        setattr(grid, "_dpf_cache", [node_coordinates, coordinates_field])

        return grid

    # might be computed when checking for VTK quadratic bug
    if offset is None:
        offset = compute_offset()

    return pv.UnstructuredGrid(offset, cells, vtk_cell_type, node_coordinates)


def dpf_mesh_to_vtk(mesh, nodes=None, as_linear=True):
    """Return a pyvista unstructured grid given DPF node and element
    definitions.

    Parameters
    ----------
    mesh : dpf.MeshedRegion
        Meshed Region to export to pyVista format

    nodes : dpf.Field, optional
        Field containing the nodes of the mesh.

    as_linear : bool, optional
        Export quadratic surface elements as linear.

    Returns
    -------
    grid : pyvista.UnstructuredGrid
        Unstructured grid of the DPF mesh.
    """
    try:
        return dpf_mesh_to_vtk_op(mesh, nodes, as_linear)
    except (AttributeError, KeyError, errors.DPFServerException):
        return dpf_mesh_to_vtk_py(mesh, nodes, as_linear)


def vtk_update_coordinates(vtk_grid, coordinates_array):
    from copy import copy

    vtk_grid.points = copy(coordinates_array)



"""
merge_data_tree
===============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class merge_data_tree(Operator):
    """Merges a list of data trees. Attributes names shouldn't be shared
    accross data tree instances.

    Parameters
    ----------
    data_tree1 : DataTree
        Either a vector of data trees or data trees
        from pin 0 to ... to merge.
    data_tree2 : DataTree
        Either a vector of data trees or data trees
        from pin 0 to ... to merge.


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.merge_data_tree()

    >>> # Make input connections
    >>> my_data_tree1 = dpf.DataTree()
    >>> op.inputs.data_tree1.connect(my_data_tree1)
    >>> my_data_tree2 = dpf.DataTree()
    >>> op.inputs.data_tree2.connect(my_data_tree2)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.merge_data_tree(
    ...     data_tree1=my_data_tree1,
    ...     data_tree2=my_data_tree2,
    ... )

    >>> # Get output data
    >>> result_any = op.outputs.any()
    """

    def __init__(self, data_tree1=None, data_tree2=None, config=None, server=None):
        super().__init__(name="merge::data_tree", config=config, server=server)
        self._inputs = InputsMergeDataTree(self)
        self._outputs = OutputsMergeDataTree(self)
        if data_tree1 is not None:
            self.inputs.data_tree1.connect(data_tree1)
        if data_tree2 is not None:
            self.inputs.data_tree2.connect(data_tree2)

    @staticmethod
    def _spec():
        description = """Merges a list of data trees. Attributes names shouldn't be shared
            accross data tree instances."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="data_tree",
                    type_names=["data_tree"],
                    optional=False,
                    document="""Either a vector of data trees or data trees
        from pin 0 to ... to merge.""",
                ),
                1: PinSpecification(
                    name="data_tree",
                    type_names=["data_tree"],
                    optional=False,
                    document="""Either a vector of data trees or data trees
        from pin 0 to ... to merge.""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="any",
                    type_names=["any"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="merge::data_tree", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMergeDataTree
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMergeDataTree
        """
        return super().outputs


class InputsMergeDataTree(_Inputs):
    """Intermediate class used to connect user inputs to
    merge_data_tree operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_data_tree()
    >>> my_data_tree1 = dpf.DataTree()
    >>> op.inputs.data_tree1.connect(my_data_tree1)
    >>> my_data_tree2 = dpf.DataTree()
    >>> op.inputs.data_tree2.connect(my_data_tree2)
    """

    def __init__(self, op: Operator):
        super().__init__(merge_data_tree._spec().inputs, op)
        self._data_tree1 = Input(merge_data_tree._spec().input_pin(0), 0, op, 0)
        self._inputs.append(self._data_tree1)
        self._data_tree2 = Input(merge_data_tree._spec().input_pin(1), 1, op, 1)
        self._inputs.append(self._data_tree2)

    @property
    def data_tree1(self):
        """Allows to connect data_tree1 input to the operator.

        Either a vector of data trees or data trees
        from pin 0 to ... to merge.

        Parameters
        ----------
        my_data_tree1 : DataTree

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_data_tree()
        >>> op.inputs.data_tree1.connect(my_data_tree1)
        >>> # or
        >>> op.inputs.data_tree1(my_data_tree1)
        """
        return self._data_tree1

    @property
    def data_tree2(self):
        """Allows to connect data_tree2 input to the operator.

        Either a vector of data trees or data trees
        from pin 0 to ... to merge.

        Parameters
        ----------
        my_data_tree2 : DataTree

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_data_tree()
        >>> op.inputs.data_tree2.connect(my_data_tree2)
        >>> # or
        >>> op.inputs.data_tree2(my_data_tree2)
        """
        return self._data_tree2


class OutputsMergeDataTree(_Outputs):
    """Intermediate class used to get outputs from
    merge_data_tree operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.merge_data_tree()
    >>> # Connect inputs : op.inputs. ...
    >>> result_any = op.outputs.any()
    """

    def __init__(self, op: Operator):
        super().__init__(merge_data_tree._spec().outputs, op)
        self._any = Output(merge_data_tree._spec().output_pin(0), 0, op)
        self._outputs.append(self._any)

    @property
    def any(self):
        """Allows to get any output of the operator

        Returns
        ----------
        my_any : Any

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.merge_data_tree()
        >>> # Connect inputs : op.inputs. ...
        >>> result_any = op.outputs.any()
        """  # noqa: E501
        return self._any



"""
compute_time_scoping
====================
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class compute_time_scoping(Operator):
    """Computes the time frequency scoping (made of set IDs) necessary to
    interpolate on a list of time or frequencies.

    Parameters
    ----------
    time_freq_values : float or Field
        List of frequencies or times needed. to
        specify load steps, put a field (and
        not a list) in input with a scoping
        located on "timefreq_steps".
    step : int, optional
    time_freq_support : TimeFreqSupport


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.utility.compute_time_scoping()

    >>> # Make input connections
    >>> my_time_freq_values = float()
    >>> op.inputs.time_freq_values.connect(my_time_freq_values)
    >>> my_step = int()
    >>> op.inputs.step.connect(my_step)
    >>> my_time_freq_support = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support.connect(my_time_freq_support)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.utility.compute_time_scoping(
    ...     time_freq_values=my_time_freq_values,
    ...     step=my_step,
    ...     time_freq_support=my_time_freq_support,
    ... )

    >>> # Get output data
    >>> result_scoping = op.outputs.scoping()
    >>> result_field = op.outputs.field()
    """

    def __init__(
        self,
        time_freq_values=None,
        step=None,
        time_freq_support=None,
        config=None,
        server=None,
    ):
        super().__init__(name="compute_time_scoping", config=config, server=server)
        self._inputs = InputsComputeTimeScoping(self)
        self._outputs = OutputsComputeTimeScoping(self)
        if time_freq_values is not None:
            self.inputs.time_freq_values.connect(time_freq_values)
        if step is not None:
            self.inputs.step.connect(step)
        if time_freq_support is not None:
            self.inputs.time_freq_support.connect(time_freq_support)

    @staticmethod
    def _spec():
        description = """Computes the time frequency scoping (made of set IDs) necessary to
            interpolate on a list of time or frequencies."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="time_freq_values",
                    type_names=["double", "vector<double>", "field"],
                    optional=False,
                    document="""List of frequencies or times needed. to
        specify load steps, put a field (and
        not a list) in input with a scoping
        located on "timefreq_steps".""",
                ),
                2: PinSpecification(
                    name="step",
                    type_names=["int32"],
                    optional=True,
                    document="""""",
                ),
                8: PinSpecification(
                    name="time_freq_support",
                    type_names=["time_freq_support"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""Time_scoping""",
                ),
                1: PinSpecification(
                    name="field",
                    type_names=["field"],
                    optional=False,
                    document="""Time_freq_values""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="compute_time_scoping", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsComputeTimeScoping
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsComputeTimeScoping
        """
        return super().outputs


class InputsComputeTimeScoping(_Inputs):
    """Intermediate class used to connect user inputs to
    compute_time_scoping operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.compute_time_scoping()
    >>> my_time_freq_values = float()
    >>> op.inputs.time_freq_values.connect(my_time_freq_values)
    >>> my_step = int()
    >>> op.inputs.step.connect(my_step)
    >>> my_time_freq_support = dpf.TimeFreqSupport()
    >>> op.inputs.time_freq_support.connect(my_time_freq_support)
    """

    def __init__(self, op: Operator):
        super().__init__(compute_time_scoping._spec().inputs, op)
        self._time_freq_values = Input(
            compute_time_scoping._spec().input_pin(0), 0, op, -1
        )
        self._inputs.append(self._time_freq_values)
        self._step = Input(compute_time_scoping._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._step)
        self._time_freq_support = Input(
            compute_time_scoping._spec().input_pin(8), 8, op, -1
        )
        self._inputs.append(self._time_freq_support)

    @property
    def time_freq_values(self):
        """Allows to connect time_freq_values input to the operator.

        List of frequencies or times needed. to
        specify load steps, put a field (and
        not a list) in input with a scoping
        located on "timefreq_steps".

        Parameters
        ----------
        my_time_freq_values : float or Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.compute_time_scoping()
        >>> op.inputs.time_freq_values.connect(my_time_freq_values)
        >>> # or
        >>> op.inputs.time_freq_values(my_time_freq_values)
        """
        return self._time_freq_values

    @property
    def step(self):
        """Allows to connect step input to the operator.

        Parameters
        ----------
        my_step : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.compute_time_scoping()
        >>> op.inputs.step.connect(my_step)
        >>> # or
        >>> op.inputs.step(my_step)
        """
        return self._step

    @property
    def time_freq_support(self):
        """Allows to connect time_freq_support input to the operator.

        Parameters
        ----------
        my_time_freq_support : TimeFreqSupport

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.compute_time_scoping()
        >>> op.inputs.time_freq_support.connect(my_time_freq_support)
        >>> # or
        >>> op.inputs.time_freq_support(my_time_freq_support)
        """
        return self._time_freq_support


class OutputsComputeTimeScoping(_Outputs):
    """Intermediate class used to get outputs from
    compute_time_scoping operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.utility.compute_time_scoping()
    >>> # Connect inputs : op.inputs. ...
    >>> result_scoping = op.outputs.scoping()
    >>> result_field = op.outputs.field()
    """

    def __init__(self, op: Operator):
        super().__init__(compute_time_scoping._spec().outputs, op)
        self._scoping = Output(compute_time_scoping._spec().output_pin(0), 0, op)
        self._outputs.append(self._scoping)
        self._field = Output(compute_time_scoping._spec().output_pin(1), 1, op)
        self._outputs.append(self._field)

    @property
    def scoping(self):
        """Allows to get scoping output of the operator

        Returns
        ----------
        my_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.compute_time_scoping()
        >>> # Connect inputs : op.inputs. ...
        >>> result_scoping = op.outputs.scoping()
        """  # noqa: E501
        return self._scoping

    @property
    def field(self):
        """Allows to get field output of the operator

        Returns
        ----------
        my_field : Field

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.utility.compute_time_scoping()
        >>> # Connect inputs : op.inputs. ...
        >>> result_field = op.outputs.field()
        """  # noqa: E501
        return self._field



"""
mesh_extraction
===============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class mesh_extraction(Operator):
    """Take a mesh and a scoping (elemental or nodal) and create a new mesh
    that contains this selection only.

    Parameters
    ----------
    mesh : MeshedRegion
    mesh_scoping : Scoping
    extension : int, optional
        Number of extension layer


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.mesh_extraction()

    >>> # Make input connections
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_extension = int()
    >>> op.inputs.extension.connect(my_extension)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.mesh_extraction(
    ...     mesh=my_mesh,
    ...     mesh_scoping=my_mesh_scoping,
    ...     extension=my_extension,
    ... )

    >>> # Get output data
    >>> result_abstract_meshed_region = op.outputs.abstract_meshed_region()
    """

    def __init__(
        self, mesh=None, mesh_scoping=None, extension=None, config=None, server=None
    ):
        super().__init__(name="mesh_extraction", config=config, server=server)
        self._inputs = InputsMeshExtraction(self)
        self._outputs = OutputsMeshExtraction(self)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)
        if mesh_scoping is not None:
            self.inputs.mesh_scoping.connect(mesh_scoping)
        if extension is not None:
            self.inputs.extension.connect(extension)

    @staticmethod
    def _spec():
        description = """Take a mesh and a scoping (elemental or nodal) and create a new mesh
            that contains this selection only."""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                0: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
                1: PinSpecification(
                    name="mesh_scoping",
                    type_names=["scoping"],
                    optional=False,
                    document="""""",
                ),
                2: PinSpecification(
                    name="extension",
                    type_names=["int32"],
                    optional=True,
                    document="""Number of extension layer""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="abstract_meshed_region",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="mesh_extraction", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsMeshExtraction
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsMeshExtraction
        """
        return super().outputs


class InputsMeshExtraction(_Inputs):
    """Intermediate class used to connect user inputs to
    mesh_extraction operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_extraction()
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    >>> my_mesh_scoping = dpf.Scoping()
    >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
    >>> my_extension = int()
    >>> op.inputs.extension.connect(my_extension)
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_extraction._spec().inputs, op)
        self._mesh = Input(mesh_extraction._spec().input_pin(0), 0, op, -1)
        self._inputs.append(self._mesh)
        self._mesh_scoping = Input(mesh_extraction._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._mesh_scoping)
        self._extension = Input(mesh_extraction._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._extension)

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_extraction()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh

    @property
    def mesh_scoping(self):
        """Allows to connect mesh_scoping input to the operator.

        Parameters
        ----------
        my_mesh_scoping : Scoping

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_extraction()
        >>> op.inputs.mesh_scoping.connect(my_mesh_scoping)
        >>> # or
        >>> op.inputs.mesh_scoping(my_mesh_scoping)
        """
        return self._mesh_scoping

    @property
    def extension(self):
        """Allows to connect extension input to the operator.

        Number of extension layer

        Parameters
        ----------
        my_extension : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_extraction()
        >>> op.inputs.extension.connect(my_extension)
        >>> # or
        >>> op.inputs.extension(my_extension)
        """
        return self._extension


class OutputsMeshExtraction(_Outputs):
    """Intermediate class used to get outputs from
    mesh_extraction operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.mesh_extraction()
    >>> # Connect inputs : op.inputs. ...
    >>> result_abstract_meshed_region = op.outputs.abstract_meshed_region()
    """

    def __init__(self, op: Operator):
        super().__init__(mesh_extraction._spec().outputs, op)
        self._abstract_meshed_region = Output(
            mesh_extraction._spec().output_pin(0), 0, op
        )
        self._outputs.append(self._abstract_meshed_region)

    @property
    def abstract_meshed_region(self):
        """Allows to get abstract_meshed_region output of the operator

        Returns
        ----------
        my_abstract_meshed_region : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.mesh_extraction()
        >>> # Connect inputs : op.inputs. ...
        >>> result_abstract_meshed_region = op.outputs.abstract_meshed_region()
        """  # noqa: E501
        return self._abstract_meshed_region



"""
from_scopings
=============
Autogenerated DPF operator classes.
"""
from warnings import warn
from ansys.dpf.core.dpf_operator import Operator
from ansys.dpf.core.inputs import Input, _Inputs
from ansys.dpf.core.outputs import Output, _Outputs
from ansys.dpf.core.operators.specification import PinSpecification, Specification


class from_scopings(Operator):
    """Extracts multiple meshed region base on a scoping and saved in a
    MeshesContainer

    Parameters
    ----------
    scopings_container : ScopingsContainer
        If nodal scoping, then the scoping is
        transposed respecting the inclusive
        pin
    inclusive : int, optional
        If inclusive == 1 then all the elements/faces
        adjacent to the nodes ids in input
        are added, if inclusive == 0, only
        the elements/faces which have all
        their nodes in the scoping are
        included
    nodes_only : bool, optional
        Returns mesh with nodes only (without any
        elements). default is false.
    mesh : MeshedRegion


    Examples
    --------
    >>> from ansys.dpf import core as dpf

    >>> # Instantiate operator
    >>> op = dpf.operators.mesh.from_scopings()

    >>> # Make input connections
    >>> my_scopings_container = dpf.ScopingsContainer()
    >>> op.inputs.scopings_container.connect(my_scopings_container)
    >>> my_inclusive = int()
    >>> op.inputs.inclusive.connect(my_inclusive)
    >>> my_nodes_only = bool()
    >>> op.inputs.nodes_only.connect(my_nodes_only)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)

    >>> # Instantiate operator and connect inputs in one line
    >>> op = dpf.operators.mesh.from_scopings(
    ...     scopings_container=my_scopings_container,
    ...     inclusive=my_inclusive,
    ...     nodes_only=my_nodes_only,
    ...     mesh=my_mesh,
    ... )

    >>> # Get output data
    >>> result_meshes = op.outputs.meshes()
    """

    def __init__(
        self,
        scopings_container=None,
        inclusive=None,
        nodes_only=None,
        mesh=None,
        config=None,
        server=None,
    ):
        super().__init__(name="meshes::by_scopings", config=config, server=server)
        self._inputs = InputsFromScopings(self)
        self._outputs = OutputsFromScopings(self)
        if scopings_container is not None:
            self.inputs.scopings_container.connect(scopings_container)
        if inclusive is not None:
            self.inputs.inclusive.connect(inclusive)
        if nodes_only is not None:
            self.inputs.nodes_only.connect(nodes_only)
        if mesh is not None:
            self.inputs.mesh.connect(mesh)

    @staticmethod
    def _spec():
        description = """Extracts multiple meshed region base on a scoping and saved in a
            MeshesContainer"""
        spec = Specification(
            description=description,
            map_input_pin_spec={
                1: PinSpecification(
                    name="scopings_container",
                    type_names=["scopings_container"],
                    optional=False,
                    document="""If nodal scoping, then the scoping is
        transposed respecting the inclusive
        pin""",
                ),
                2: PinSpecification(
                    name="inclusive",
                    type_names=["int32"],
                    optional=True,
                    document="""If inclusive == 1 then all the elements/faces
        adjacent to the nodes ids in input
        are added, if inclusive == 0, only
        the elements/faces which have all
        their nodes in the scoping are
        included""",
                ),
                3: PinSpecification(
                    name="nodes_only",
                    type_names=["bool"],
                    optional=True,
                    document="""Returns mesh with nodes only (without any
        elements). default is false.""",
                ),
                7: PinSpecification(
                    name="mesh",
                    type_names=["abstract_meshed_region"],
                    optional=False,
                    document="""""",
                ),
            },
            map_output_pin_spec={
                0: PinSpecification(
                    name="meshes",
                    type_names=["meshes_container"],
                    optional=False,
                    document="""""",
                ),
            },
        )
        return spec

    @staticmethod
    def default_config(server=None):
        """Returns the default config of the operator.

        This config can then be changed to the user needs and be used to
        instantiate the operator. The Configuration allows to customize
        how the operation will be processed by the operator.

        Parameters
        ----------
        server : server.DPFServer, optional
            Server with channel connected to the remote or local instance. When
            ``None``, attempts to use the global server.
        """
        return Operator.default_config(name="meshes::by_scopings", server=server)

    @property
    def inputs(self):
        """Enables to connect inputs to the operator

        Returns
        --------
        inputs : InputsFromScopings
        """
        return super().inputs

    @property
    def outputs(self):
        """Enables to get outputs of the operator by evaluating it

        Returns
        --------
        outputs : OutputsFromScopings
        """
        return super().outputs


class InputsFromScopings(_Inputs):
    """Intermediate class used to connect user inputs to
    from_scopings operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.from_scopings()
    >>> my_scopings_container = dpf.ScopingsContainer()
    >>> op.inputs.scopings_container.connect(my_scopings_container)
    >>> my_inclusive = int()
    >>> op.inputs.inclusive.connect(my_inclusive)
    >>> my_nodes_only = bool()
    >>> op.inputs.nodes_only.connect(my_nodes_only)
    >>> my_mesh = dpf.MeshedRegion()
    >>> op.inputs.mesh.connect(my_mesh)
    """

    def __init__(self, op: Operator):
        super().__init__(from_scopings._spec().inputs, op)
        self._scopings_container = Input(from_scopings._spec().input_pin(1), 1, op, -1)
        self._inputs.append(self._scopings_container)
        self._inclusive = Input(from_scopings._spec().input_pin(2), 2, op, -1)
        self._inputs.append(self._inclusive)
        self._nodes_only = Input(from_scopings._spec().input_pin(3), 3, op, -1)
        self._inputs.append(self._nodes_only)
        self._mesh = Input(from_scopings._spec().input_pin(7), 7, op, -1)
        self._inputs.append(self._mesh)

    @property
    def scopings_container(self):
        """Allows to connect scopings_container input to the operator.

        If nodal scoping, then the scoping is
        transposed respecting the inclusive
        pin

        Parameters
        ----------
        my_scopings_container : ScopingsContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.from_scopings()
        >>> op.inputs.scopings_container.connect(my_scopings_container)
        >>> # or
        >>> op.inputs.scopings_container(my_scopings_container)
        """
        return self._scopings_container

    @property
    def inclusive(self):
        """Allows to connect inclusive input to the operator.

        If inclusive == 1 then all the elements/faces
        adjacent to the nodes ids in input
        are added, if inclusive == 0, only
        the elements/faces which have all
        their nodes in the scoping are
        included

        Parameters
        ----------
        my_inclusive : int

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.from_scopings()
        >>> op.inputs.inclusive.connect(my_inclusive)
        >>> # or
        >>> op.inputs.inclusive(my_inclusive)
        """
        return self._inclusive

    @property
    def nodes_only(self):
        """Allows to connect nodes_only input to the operator.

        Returns mesh with nodes only (without any
        elements). default is false.

        Parameters
        ----------
        my_nodes_only : bool

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.from_scopings()
        >>> op.inputs.nodes_only.connect(my_nodes_only)
        >>> # or
        >>> op.inputs.nodes_only(my_nodes_only)
        """
        return self._nodes_only

    @property
    def mesh(self):
        """Allows to connect mesh input to the operator.

        Parameters
        ----------
        my_mesh : MeshedRegion

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.from_scopings()
        >>> op.inputs.mesh.connect(my_mesh)
        >>> # or
        >>> op.inputs.mesh(my_mesh)
        """
        return self._mesh


class OutputsFromScopings(_Outputs):
    """Intermediate class used to get outputs from
    from_scopings operator.

    Examples
    --------
    >>> from ansys.dpf import core as dpf
    >>> op = dpf.operators.mesh.from_scopings()
    >>> # Connect inputs : op.inputs. ...
    >>> result_meshes = op.outputs.meshes()
    """

    def __init__(self, op: Operator):
        super().__init__(from_scopings._spec().outputs, op)
        self._meshes = Output(from_scopings._spec().output_pin(0), 0, op)
        self._outputs.append(self._meshes)

    @property
    def meshes(self):
        """Allows to get meshes output of the operator

        Returns
        ----------
        my_meshes : MeshesContainer

        Examples
        --------
        >>> from ansys.dpf import core as dpf
        >>> op = dpf.operators.mesh.from_scopings()
        >>> # Connect inputs : op.inputs. ...
        >>> result_meshes = op.outputs.meshes()
        """  # noqa: E501
        return self._meshes
